
use constant false => 0;
use constant true => 1;
use constant CHILD_PROCESS_STARTED => 1;
use constant CHILD_PROCESS_COMPLETED => 2;
use constant LIMIT => 50*1024;

my $currentFilePath  = __FILE__;
require substr($currentFilePath, 0, rindex($currentFilePath, '/')) . '/fetchPath.pl';

my $jobrunningDir = `pwd`;
chomp($jobrunningDir);
my $lineFeed = "\n";
my $operationComplete = "100%";

# File containing the status of Operation
my $StatusFilePath = "./STATUS_FILE";

# Temporary directory created by idevsutil binary           
my $evsTempDirPath = "./evs_temp";

# Index number for statusFileArray
use constant BACKUP_COUNT_FILES_INDEX => 0;
use constant SYNC_COUNT_FILES_INDEX => 1;
use constant FAILED_COUNT_FILES_INDEX => 2;
use constant FILE_COUNT_INDEX => 3;
use constant EXIT_FLAG_INDEX => 4;

my $exit_flag = 0;
my $skip = 0;
my $fileBackupCount = 0;
my $fileSyncCount = 0;
my $failedfiles_count = 0;
my $failedfiles_index = 0;
my $sleep_count = 0;
my %current_BackupsetHash = 0;
my $retry_info = "./retry_info";
my $retryAttempt = false;
my @failedfiles_array = ();

# parameters sent with the script calling
my $outputFilePath = $ARGV[0];							   
my $current_backupsetfile = $ARGV[1];
my $relative = $ARGV[2];
my $current_source = $ARGV[3];
my $backup_error = $current_backupsetfile."_ERROR";

# Errors encountered during backup operation for which the script should not retry the backup operation                         
my @ErrorArgumentsNoRetry = ("No such file or directory",
                             "File name too long",
							 "skipping non-regular file",
							 "Permission Denied",
							 "SFERROR",
                             "IOERROR",
                             "mkstemp"
							);	
							
# Errors encountered during backup operation for which the script should not retry the backup operation                         
my @ErrorArgumentsExit = (  "encryption verification failed",
                             "some files could not be transferred due to quota over limit",
                             "skipped-over limit",
                             "account is under maintenance",
                             "account has been cancelled",
                             "account has been expired",
                             "protocol version mismatch",
                             "password mismatch",
                             "out of memory"
                            );
						
# Errors encountered during backup operation for which the script should retry the backup operation                          
my @ErrorArgumentsRetry = ("io timeout",
                           "Operation timed out",
                           "nodename nor servname provided, or not known",
                           "failed to connect",
						   "connection unexpectedly closed",
                           "Connection refused",
						   "failed verification -- update",
						   "unauthorized user"
                          );

undef @linesStatusFile;
if(!open(CBFH, "<", "$current_backupsetfile")){
	traceLog("Unable to open current backup set file $current_backupsetfile, Reason: $!", __FILE__, __LINE__);
	exit 0;
}
@currentBackupset = <CBFH>;
close CBFH;

for (my $i=0; $i <= $#currentBackupset; $i++){
	chomp $currentBackupset[$i];
	$current_BackupsetHash{$currentBackupset[$i]} = 0;
}		

# signal handlers
$SIG{INT} = \&process_term;
$SIG{TERM} = \&process_term;

#----------------- Main -----------------------------
subOutputProcess();
subErrorRoutine();
writeParameterValuesToStatusFile();
#-----------------------------------------------------

#****************************************************************************************************
# Subroutine Name         : subOutputProcess.
# Objective               : This function monitors and parse the evs output file and creates the App 
#							log file which is shown to user. 					
# Modified By             : Deepak Chaurasia
#*****************************************************************************************************/
sub subOutputProcess()
{
	# Flags to determine the status of OUTFILE 
	my $fileOpened = 0;
	my $termData = "CHILD_PROCESS_COMPLETED";
	my $buffer = "";
	my $byteRead = 0;
	my $lastLine  = "";
	my $fields_in_progress = 10;
	
	# Waiting for EVS output file to open
	while(!$fileOpened){
		if(!-e $processIdFile){
			last;
		}
		open TEMPOUTPUTFILE, "<", $OutErrorFilePaths[0] and $fileOpened = 1;
		sleep(2);
	}
	
	if(!$fileOpened){
		traceLog("Unable to open file $OutErrorFilePaths[0], Reason: $!", __FILE__, __LINE__);
		return;
	}

	# App log file open for parsed output
	if(!open(OUTFILE, ">> $outputFilePath")){
		traceLog("Unable to open file $outputFilePath, Reason: $!", __FILE__, __LINE__);
		return;
	}
	
	my $Backuptype ="";
	while (1) {
		$byteRead = read(TEMPOUTPUTFILE, $buffer, LIMIT);
		if($byteRead == 0){
			if(!-e $processIdFile){
				last;
			}
			
			sleep(2);
			seek(TEMPOUTPUTFILE, 0, 1);		#to clear eof flag
			next;
		}
	
		if("" ne $lastLine){			# need to check appending partial record to packet or to first line of packet
			$buffer = $lastLine . $buffer;
		}
		
		my @resultList = split /\n/, $buffer;
		my $bufIndex = @resultList;
	
		if($buffer !~ /\n$/){       #keep last line of buffer only when it not ends with newline.
			$lastLine = $resultList[$#resultList];
			$bufIndex -= 1;
		}
		else{
			$lastLine = "";
		}
		
		for(my $cnt = 0; $cnt < $bufIndex; $cnt++) 
		{
			my $tmpLine = $resultList[$cnt];
			chomp($tmpLine);
			$tmpLine =~ s/^\s+|\s+$//g;
		
			# check if this is 100% progress line
			if($tmpLine =~ m/$operationComplete/) {
				my @fields = split("\\] \\[",$tmpLine,$fields_in_progress);
				my $total_fields = @fields;
				
				if($total_fields == $fields_in_progress) {				
					# remove starting character [ from first field
					$fields[0] =~ s/^.//;

					# remove last character ] from last field
					$fields[$fields_in_progress-1] =~ s/.$//;

					# remove spaces from beginning from required fields
					$fields[0] =~ s/^\s+//;
					$fields[$fields_in_progress-2] =~ s/^\s+//;
					my $keyString = $pathSeparator.$fields[$fields_in_progress-1];
					
					if($fields[$fields_in_progress-2] eq "FILE IN SYNC"){ 					# check if file in sync
						if(defined($current_BackupsetHash{$keyString})){
							$current_BackupsetHash{$keyString} = 1;
						}
						else{
							getFullPathofFile($keyString);
						}
						$fileSyncCount++;
						next;
					}
					elsif($fields[$fields_in_progress-2] eq "FULL"){
						if(defined($current_BackupsetHash{$keyString})){
							$current_BackupsetHash{$keyString} = 1;
						}
						else{	
							getFullPathofFile($keyString);
						}
						$fileBackupCount++;
						$Backuptype = "Full Backup";
					}
					elsif($fields[$fields_in_progress-2] eq "INCREMENTAL"){
						if(defined($current_BackupsetHash{$keyString})){
							$current_BackupsetHash{$keyString} = 1;
						}
						else{	
							getFullPathofFile($keyString);
						}
						$fileBackupCount++;
						$Backuptype = "Incremental Backup";
					}
					else{
						if($tmpLine ne ""){
							addtionalErrorInfo(\$tmpLine,$backup_error);
						}
					}
					
					my $backupFinishTime = localtime();
					my $fileSize = convertFileSize($fields[0]);         
					my $filename = $fields[$fields_in_progress-1];
					if($current_backupsetfile =~ m/_onlyFiles/){
						$pos = rindex ($filename, '/');
						$filename = substr($filename,$pos+1);
					}
					print OUTFILE "[$backupFinishTime] [$Backuptype] [SUCCESS] [$pathSeparator$filename] [$fileSize] $lineFeed";
				}
				else{
					if($tmpLine ne ""){
						addtionalErrorInfo(\$tmpLine,$backup_error);
					}
				}
			}
			elsif($tmpLine =~ $termData) 
			{
				unlink($OutErrorFilePaths[0]);
				$skip = 1;
				last;
			}
			elsif($tmpLine !~ m/building file list/ and $tmpLine !~ m/=============/ and $tmpLine !~ m/connection established/ and $tmpLine !~ m/bytes  received/ and $tmpLine !~ m/\| FILE SIZE \| TRANSFERED SIZE \| TOTAL SIZE \|/){
				if($tmpLine ne ''){
					addtionalErrorInfo(\$tmpLine,$backup_error);
				}
			}
		}
		if($skip == 1){
			$skip = 0;
			last;
		}
		#select(undef, undef, undef, 0.50);
	}
	
	close OUTFILE;
	close TEMPOUTPUTFILE;
	chmod $permissionVal, $outputFilePath;
}

#****************************************************************************************************
# Subroutine Name         : subErrorRoutine.
# Objective               : This function monitors and parse the evs output file and creates the App 
#							log file which is shown to user. 					
# Modified By             : Deepak Chaurasia
#*****************************************************************************************************/
sub subErrorRoutine()
{
	getFailedFileCount();
	#Copying evs generated error.txt to BACKUP_ERRORFILE
	copyTempErrorFile($backup_error);
	getParameterValueFromStatusFile(\$statusFileArray[FILE_COUNT_INDEX],\$failedfiles_index);

	my $individual_errorfile = $jobrunningDir."/ERROR/$backup_error";
	if(-s $individual_errorfile > 0 and $failedfiles_index != -1){
		if(!checkretryAttempt($individual_errorfile)){
			updateFailedFileCount();
			getFinalErrorFile();
			return;
		}
	}
	else{
		updateFailedFileCount();
		getFinalErrorFile();
		return;
	}

	if(!$retryAttempt or $failedfiles_index == -1){
		getFinalErrorFile();
	}
	else{
		getParameterValueFromStatusFile(\$statusFileArray[FILE_COUNT_INDEX],\$failedfiles_index);
		if($#failedfiles_array >= 0){
			$failedfiles_index++;
			$failedfiles = "failedfiles.txt".$failedfiles_index;
			my $oldfile_error = $jobrunningDir."/ERROR/$backup_error";
			my $newfile_error = $jobrunningDir."/ERROR/$failedfiles"."_ERROR";
			
			if(-e $oldfile_error or -e $oldfile_error."_FINAL"){
				rename $oldfile_error, $newfile_error;
			}
			if(!open(FAILEDLIST, "> $failedfiles")){
				traceLog("Unable to open file $failedfiles, Reason:$!", __FILE__, __LINE__);
				return;
			}

			for(my $i = 0; $i <= $#failedfiles_array; $i++){
				print FAILEDLIST "$failedfiles_array[$i]\n";
			}
			close FAILEDLIST;
		}
		
		if(-e $failedfiles){
			open RETRYINFO, ">> $retryinfo";
			print RETRYINFO "$failedfiles $relative $current_source\n";
			close RETRYINFO;
		}
	}

	updateFailedFileCount();
}

#****************************************************************************************************
# Subroutine Name         : checkretryAttempt.
# Objective               : This function checks whether backup has to retry              
# Added By				  : Pooja Havaldar
#*****************************************************************************************************/
sub checkretryAttempt
{
	my $errorline = "idevs error";
	my $individual_errorfile = $_[0];
	
	#check for retry attempt
	if(!open(TEMPERRORFILE, "< $individual_errorfile")){
		traceLog("Unable to open file $individual_errorfile, Reason:$!", __FILE__, __LINE__);
		return 0;
	}
	
	@linesBackupErrorFile = <TEMPERRORFILE>;
	close TEMPERRORFILE;
	chomp(@linesBackupErrorFile);
	#To check the individual error file for exit scenarios
	for(my $i = 0; $i <= $#linesBackupErrorFile; $i++){
		$linesBackupErrorFile[$i] =~ s/^\s+|\s+$//g;
		if($linesBackupErrorFile[$i] eq "" or $linesBackupErrorFile[$i] =~ m/$errorline/){
			next;
		}
		
		for(my $j=0; $j<=$#ErrorArgumentsExit; $j++){
			if($linesBackupErrorFile[$i] =~ m/$ErrorArgumentsExit[$j]/){
				unlink($OutErrorFilePaths[0]);
				if("" ne $evsTempDirPath){
					`rm -rf $evsTempDirPath`;
				}

				traceLog("Operation could not be completed. Reason : $ErrorArgumentsExit[$j]", __FILE__, __LINE__);
				$errStr = "Operation could not be completed. Reason : $ErrorArgumentsExit[$j] \n";
				#kill evs and then exit
				$fileJobTerminationPath = "$baseDir/$webDir/$APPDIR/.scripts/Job_Termination_Script.pl"; 
				`$perlPath $fileJobTerminationPath`;
				$exit_flag = "1-$errStr";
				return 0;
			}
		}	
	}
	
	#To check the individual error file for retry case
	for(my $i = 0; $i<= $#linesBackupErrorFile; $i++){
		$linesBackupErrorFile[$i] =~ s/^\s+|\s+$//g;
		if($linesBackupErrorFile[$i] eq "" or $linesBackupErrorFile[$i] =~ m/$errorline/){
			next;
		}
		
		for(my $j=0; $j<=$#ErrorArgumentsRetry; $j++){
			if($linesBackupErrorFile[$i] =~ m/$ErrorArgumentsRetry[$j]/){
				$retryAttempt = true;
				last;
			}
		}	
		if($retryAttempt){
			last;
		}
	}
	return 1;
}

#****************************************************************************************************
# Subroutine Name         : process_term
# Objective               : The signal handler invoked when SIGTERM signal is received by the script       
# Created By              : Arnab Gupta
#*****************************************************************************************************/
sub process_term()
{
  writeParameterValuesToStatusFile();
  exit 0;
}

#****************************************************************************************************
# Subroutine Name         : writeParameterValuesToStatusFile
# Objective               : This subroutine writes the value of the specified parameter to the Status File 
# Modified By             : Deepak Chaurasia
#*****************************************************************************************************/
sub writeParameterValuesToStatusFile()
{
	my $Backupcount= 0;
	my $Synccount = 0;
	my $failedcount = 0;
	
	# read the backup, sync and error count from status file
	getParameterValueFromStatusFile(\$statusFileArray[BACKUP_COUNT_FILES_INDEX],\$Backupcount);
	getParameterValueFromStatusFile(\$statusFileArray[SYNC_COUNT_FILES_INDEX],\$Synccount);
	getParameterValueFromStatusFile(\$statusFileArray[FAILED_COUNT_FILES_INDEX],\$failedcount);

	if(!open(STATUS_FILE, "> $StatusFilePath")){
		traceLog("Unable to open file $StatusFilePath, Reason:$!", __FILE__, __LINE__);
		return;
	}
	
	#autoflush STATUS_FILE;
	select((select(STATUS_FILE), $| = 1)[0]);
	
	# Calculate the backup, sync and error count based on new values
	$Backupcount += $fileBackupCount;
	$Synccount += $fileSyncCount;
	$failedcount = $failedfiles_count;
	
	print STATUS_FILE 	"$statusFileArray[BACKUP_COUNT_FILES_INDEX] = $Backupcount \n",
						"$statusFileArray[SYNC_COUNT_FILES_INDEX] = $Synccount \n",
						"$statusFileArray[FAILED_COUNT_FILES_INDEX] = $failedcount \n",
						"$statusFileArray[FILE_COUNT_INDEX] = $failedfiles_index \n",
						"$statusFileArray[EXIT_FLAG_INDEX] = $exit_flag ";
	close STATUS_FILE;
	chmod $permissionVal, $StatusFilePath;
	undef @linesStatusFile;
}

#****************************************************************************************************************************
# Subroutine Name         : getFullPathofFile
# Objective               : This subroutine gets the full path of files that have backed up 
# Modified By             : Pooja Havaldar
#******************************************************************************************************************************/
sub getFullPathofFile()
{
	$fileToCheck = $_[0];
	for(my $i = $#currentBackupset ; $i >= 0; $i--){
		$a = rindex ($currentBackupset[$i], '/');
		$match = substr($currentBackupset[$i],$a);
		if($fileToCheck eq $match){
			$current_BackupsetHash{$currentBackupset[$i]} = 1;
			last;
		}
	}
}

#****************************************************************************************************************************
# Subroutine Name         : getFinalErrorFile
# Objective               : This subroutine creates a final ERROR file for each backupset, which has to be displayed in LOGS  
# Modified By             : Pooja Havaldar
#******************************************************************************************************************************/
sub getFinalErrorFile()
{	
	$cancel = 0;
	if($exit_flag == 0){
		if(!-e $processIdFile){
			$cancel = 1;
		}
	}
	my $failedWithReason = 0;
	my $errFinal = "";
	my $errMsgFinal = "";
	my $fileOpenFlag = 1;
	my $individual_errorfile = $jobrunningDir."/ERROR/$backup_error";

	if($failedfiles_count > 0){
		open ERROR_FINAL, ">", $individual_errorfile."_FINAL" or $fileOpenFlag = 0;
		if(!$fileOpenFlag){
			traceLog("Unable to open $individual_errorfile"."_FINAL, Reason:$!", __FILE__, __LINE__);
			return;
		}
		else{
			open BACKUP_ERROR, "<", $individual_errorfile or $fileOpenFlag = 0;
			if(!$fileOpenFlag){
				if($exit_flag != 0 or $cancel != 0){
					$failedfiles_count = $failedWithReason;
					#traceLog("Unable to open $individual_errorfile, Reason:$!", __FILE__, __LINE__);
				}
				return;
			}
			@individual_errorfileContents = <BACKUP_ERROR>;
			close BACKUP_ERROR;
		}
		chomp(@failedfiles_array);
		chomp(@individual_errorfileContents);
		
		my $j = 0; 
		@failedfiles_array = sort @failedfiles_array;
		my $last_index = $#individual_errorfileContents;
	
		for($i = 0; $i <= $#failedfiles_array; $i++){
			$matched = 0;
			
			#reset the initial and last limit for internal for loop for new item if required
			if($j > $last_index){
				$j = 0;
				$last_index = $#individual_errorfileContents;
			}
			
			#fill the last matched index for later use
			$index = $j;
			$failedfile = substr($failedfiles_array[$i],1);
			$failedfile = quotemeta($failedfile);
			
			#try to find a match between start and end point of error file
			for(;$j <= $last_index; $j++){
				if($individual_errorfileContents[$j] =~ /$failedfile/){
					$individual_errorfileContents[$j] =~ s/$failedfile//;
					print ERROR_FINAL "[".localtime()."] [FAILED] [$failedfiles_array[$i]] Reason : $individual_errorfileContents[$j] $lineFeed";
					$matched = 1;
					$failedWithReason++;	
						
					#got a match so resetting the last index 
					$last_index = $#individual_errorfileContents;
					$j++;
					last;
				}
				
				#if no match till last item and intial index is not zero try for remaining error file content
				if($j == $last_index && $index != 0){
					$j = 0;
					$last_index = $index;
					$index = $j;
				}
			}
			
			if($matched == 0 and $exit_flag == 0 and $cancel == 0){
				print ERROR_FINAL "[".localtime()."] [FAILED] [$failedfiles_array[$i]] $lineFeed";
			}
		}
		
		close ERROR_FINAL;
	}
	
	if($exit_flag != 0 or $cancel != 0){
		$failedfiles_count = $failedWithReason;
	}
	unlink $individual_errorfile;
}

#****************************************************************************************************************************
# Subroutine Name         : getFailedFileCount
# Objective               : This subroutine gets the failed files count from the failedfiles array  
# Modified By             : Pooja Havaldar
#******************************************************************************************************************************/
sub getFailedFileCount()
{
	my $j = 0;
	for(my $i = 0; $i <= $#currentBackupset; $i++){
		chomp $currentBackupset[$i];
		if($current_BackupsetHash{$currentBackupset[$i]} == 0){
			$failedfiles_array[$j] = $currentBackupset[$i];
			$j++;
		}
	}
	$failedfiles_count = $j;
}

#****************************************************************************************************************************
# Subroutine Name         : updateFailedFileCount
# Objective               : This subroutine gets the updated failed files count incase retry backup is in process  
# Modified By             : Pooja Havaldar
#******************************************************************************************************************************/
sub updateFailedFileCount()
{
	getParameterValueFromStatusFile(\$statusFileArray[FAILED_COUNT_FILES_INDEX],\$orig_count);
	if($backup_error =~ m/failedfiles.txt/){
		$size_Backupset = $#currentBackupset+1;
		$newcount = $orig_count - $size_Backupset + $failedfiles_count;
		$failedfiles_count = $newcount;
	}else{
		$orig_count += $failedfiles_count;
		$failedfiles_count = $orig_count;
	}
}
